//------------------------------------------------------- 2021-01-07 -----------
// TITLE   :Acoustic Target Systems
// FILE    :electric_target.c
// LANGUAGE:microchip xc16
// IDE     :MPLAB X IDE v3.55
// MPU     :PIC24FJ64GA002 
//-------------------------------- Written by Hiroyuki Iizuka ---- Japan -------
#define p24FJ64GA002

#define FOSC 32000000   
#define FCY  (FOSC/2)

#define USE_AND_OR

#include <xc.h>
#include <stdio.h>
#include <stdbool.h>
#include <libpic30.h>
#include <uart.h>
#include <PPS.h>
#include <InCap.h>
#include <timer.h>

#pragma config POSCMOD  = HS
#pragma config FNOSC    = PRIPLL
#pragma config OSCIOFNC = OFF
#pragma config FWDTEN   = OFF
#pragma config FCKSM    = CSDCMD
#pragma config IESO     = OFF
#pragma config WUTSEL   = LEG
#pragma config JTAGEN   = OFF

#define BAUD_RATE    19200
#define UxBRG_ARG  (((FCY/BAUD_RATE)/16)-1)   

#define PORT_INPUT  1
#define PORT_OUTPUT 0

#define NUMBER_OF_MIC     4

#define MAX_TIMER_VALUE 0xFFFF

#define OW_HIGH  1
#define OW_LOW   0

#define OW_SET    1
#define OW_CLEAR  0

#define OW_LAT        LATBbits.LATB3
#define OW_PORT       PORTBbits.RB3
#define OW_TRIS       TRISBbits.TRISB3

#define RELAY1_LAT    LATBbits.LATB11
#define RELAY1_TRIS   TRISBbits.TRISB11

#define RELAY2_LAT    LATBbits.LATB10
#define RELAY2_TRIS   TRISBbits.TRISB10

#define SWITCH1_PORT  PORTBbits.RB4
#define SWITCH1_TRIS  TRISBbits.TRISB4

#define SWITCH2_PORT  PORTABbits.RA4
#define SWITCH2_TRIS  TRISAbits.TRISA4

#define LED1_LAT      LATBbits.LATB15
#define LED1_TRIS     TRISBbits.TRISB15

#define MAX485_LAT    LATBbits.LATB12
#define MAX485_TRIS   TRISBbits.TRISB12

#define DS18B20_RESET_ERROR 0x0800

#define ANALOG_INPUT_OFF 0xFFFF

#define COMMAND_QUERY_TEMPERATURE     '1'
#define COMMAND_LED1_ON               '2'
#define COMMAND_LED1_OFF              '3'
#define COMMAND_RELAY1_ON             '4'
#define COMMAND_RELAY1_OFF            '5'
#define COMMAND_RELAY2_ON             '6'
#define COMMAND_RELAY2_OFF            '7'

//------------------------------------------------------------------------------
//  serial port driver output enable
//------------------------------------------------------------------------------
void rs485_TX_enable()
{
	while( BusyUART1() ) { ; } 	
  MAX485_LAT = 1;
}

//------------------------------------------------------------------------------
//  serial port receiver output enable
//------------------------------------------------------------------------------
void rs485_RX_enable()
{
	while( BusyUART1() ) { ; } 
  MAX485_LAT = 0;
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
void led_on(void)
{
	LED1_LAT = 1;
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
void led_off(void)
{
	LED1_LAT = 0;
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
void relay1_on(void)
{
	RELAY1_LAT = 1;
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
void relay1_off(void)
{
	RELAY1_LAT = 0;
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
void relay2_on(void)
{
	RELAY2_LAT = 1;
}

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
void relay2_off(void)
{
	RELAY2_LAT = 0;
}

//------------------------------------------------------------------------------
//  1-Wire drive High
//------------------------------------------------------------------------------
void drive_OW_high(void)
{
	OW_TRIS = PORT_OUTPUT;
	OW_LAT  = OW_HIGH;	
}

//------------------------------------------------------------------------------
//  1-Wire drive Low
//------------------------------------------------------------------------------
void drive_OW_low(void)
{
	OW_TRIS = PORT_OUTPUT;
	OW_LAT  = OW_LOW;
}

//------------------------------------------------------------------------------
//  1-Wire drive input mode
//------------------------------------------------------------------------------
void drive_OW_input(void)
{
	OW_TRIS = PORT_INPUT;
}

//------------------------------------------------------------------------------
//  1-Wire read port
//------------------------------------------------------------------------------
unsigned char read_OW(void)
{
	unsigned char read_data;
	
	OW_TRIS = PORT_INPUT;

	if(OW_PORT == OW_HIGH) {
	 	read_data = OW_SET;
	} else { 	
		read_data = OW_CLEAR;
	}
	
	return read_data;
}

//------------------------------------------------------------------------------
//  1-Wire reset
//  return 0:connection error
//         1:sensor is present
//------------------------------------------------------------------------------
bool OW_Reset(void) {
	unsigned char presence_detect;
	
	drive_OW_low();
  __delay_us(480);
  drive_OW_input();
  __delay_us(480);		
	presence_detect = read_OW();
  __delay_us(410);		
	
  return presence_detect;
}

//------------------------------------------------------------------------------
//  1-Wire write bit
//------------------------------------------------------------------------------
void OW_write_bit (unsigned char write_bit)
{
	if( write_bit ){	  //writing a bit '1'
		drive_OW_low(); 			// Drive the bus low
		__delay_us(6);				// delay 6 microsecond (us)
		drive_OW_high(); 			// Release the bus
		__delay_us(64);				// delay 64 microsecond (us)
	}	else {           //writing a bit '0'
		drive_OW_low(); 				// Drive the bus low
		__delay_us(60);					// delay 60 microsecond (us)
		drive_OW_high();  			// Release the bus
		__delay_us(10);					// delay 10 microsecond for recovery (us)
	}
}	

//------------------------------------------------------------------------------
//  1-Wire read bit
//------------------------------------------------------------------------------
unsigned char OW_read_bit (void)
{
	unsigned char read_data; 
	//reading a bit 
	drive_OW_low(); 			 // Drive the bus low
	__delay_us(6)					 // delay 6 microsecond (us)
	drive_OW_high();  	   // Release the bus
	__delay_us(9)					 // delay 9 microsecond (us)

	read_data = read_OW(); //Read the status of OW_PIN

	__delay_us(55);				 // delay 55 microsecond (us)	
	return read_data;
}

//------------------------------------------------------------------------------
//  1-Wire write byte
//------------------------------------------------------------------------------
void OW_write_byte(unsigned char write_data)
{
	unsigned char loop;
	
	for( loop = 0 ; loop < 8 ; loop++ )	{
		OW_write_bit(write_data & 0x01); 	//Sending LS-bit first
		write_data >>= 1;               	// shift the data byte for the next bit to send
	}	
}	

//------------------------------------------------------------------------------
//  1-Wire read byte
//------------------------------------------------------------------------------
unsigned char OW_read_byte (void)
{
	unsigned char result = 0;
	unsigned char loop;
	
	for( loop = 0 ; loop < 8 ; loop++ ) {
		result >>= 1; 				// shift the result to get it ready for the next bit to receive
		if( OW_read_bit() ) {
		  result |= 0x80;				// if result is one, then set MS-bit
		}	
	}
	return result;					
}	

//------------------------------------------------------------------------------
// DALLAS(MAXIM) DS18B20  1-Wire Digital Thermometer 
//------------------------------------------------------------------------------
int ds18b20_read(void)
{
	uint16_t raw_temp_value;
	
  if(OW_Reset() == 0) {   // send start pulse
    return DS18B20_RESET_ERROR;           
  }

  OW_write_byte(0xCC);   // send skip ROM command
  OW_write_byte(0x44);   // send start conversion command
 
  while(OW_read_byte() == 0) { ; }  // wait for conversion complete
 
  if(OW_Reset() == 0) {   // send start pulse
    return DS18B20_RESET_ERROR;            
  }
	
  OW_write_byte(0xCC);  // send skip ROM command
  OW_write_byte(0xBE);  // send read command
  
  raw_temp_value  = OW_read_byte();         // read temperature LSB byte and store it on raw_temp_value LSB byte
  raw_temp_value |= (OW_read_byte() << 8);  // read temperature MSB byte and store it on raw_temp_value MSB byte
 
  return (int)raw_temp_value; 
}

//------------------------------------------------------------------------------
// main
//------------------------------------------------------------------------------
int main(void) {
	unsigned int timer_value_mic[NUMBER_OF_MIC];
	unsigned int timer_value_mic_dummy;
	bool         sound_capture_mic[NUMBER_OF_MIC];
	int i;
	bool sound_capture;
	bool error_microphone;
	char buf[100];
	unsigned int max_capture_value;
	unsigned int min_capture_value;
	int raw_tmperature;
	unsigned int command;
	
	__delay_ms(300);
	
  MAX485_TRIS = PORT_OUTPUT;  // MAX485 RE/DE
	LED1_TRIS   = PORT_OUTPUT;  // LED
  SWITCH1_TRIS = PORT_INPUT;
  SWITCH2_TRIS = PORT_INPUT;
  RELAY1_TRIS = PORT_OUTPUT;
  RELAY2_TRIS = PORT_OUTPUT;
					
	AD1PCFG = ANALOG_INPUT_OFF;
	
  iPPSOutput( OUT_PIN_PPS_RP13, OUT_FN_PPS_U1TX );
  iPPSInput(  IN_FN_PPS_U1RX,  IN_PIN_PPS_RP14  ); 

 	iPPSInput(IN_FN_PPS_IC1,IN_PIN_PPS_RP5);  
 	iPPSInput(IN_FN_PPS_IC2,IN_PIN_PPS_RP6);  
 	iPPSInput(IN_FN_PPS_IC3,IN_PIN_PPS_RP7);  
 	iPPSInput(IN_FN_PPS_IC4,IN_PIN_PPS_RP8);  

	OpenTimer2( T2_ON | T2_PS_1_1 | T2_SOURCE_INT, 0xFFFF );
	
	DisableIntIC1;
	DisableIntIC2;
	DisableIntIC3;
	DisableIntIC4;
	OpenCapture1(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
	OpenCapture2(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
	OpenCapture3(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
	OpenCapture4(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
	
	ConfigIntUART1(UART_RX_INT_DIS & UART_TX_INT_DIS );
  OpenUART1( UART_EN 
           | UART_DIS_ABAUD
				   | UART_NO_PAR_8BIT
					 | UART_1STOPBIT 
					  , UART_TX_ENABLE, UxBRG_ARG );
	
	rs485_RX_enable();	
	
	__delay_ms(200);
	
	while(U1STAbits.URXDA == 1) {   // clear serial buffer 
    char Temp;
    Temp = U1RXREG;
  }
	
	sound_capture    = false;
	error_microphone = false;

	led_on();		
	while( true ) {
		if( DataRdyUART1() == 1 ) {
		  command = ReadUART1();
		  switch( command ) {
				case COMMAND_QUERY_TEMPERATURE:
					raw_tmperature = ds18b20_read();
		      sprintf(buf, "%d\n",  raw_tmperature );
					rs485_TX_enable();
		      putsUART1( (unsigned int *)buf );
					rs485_RX_enable();
					break;
				case COMMAND_LED1_ON:
					led_on();
					break;
				case COMMAND_LED1_OFF:
					led_off();
					break;
				case COMMAND_RELAY1_ON:
					relay1_on();
					break;
				case COMMAND_RELAY1_OFF:
					relay1_off();
					break;
				case COMMAND_RELAY2_ON:
					relay2_on();
					break;
				case COMMAND_RELAY2_OFF:
					relay2_off();
					break;
			}
		}	
		
		if( IC1CONbits.ICBNE ) { sound_capture = true; }
		if( IC2CONbits.ICBNE ) { sound_capture = true; }
		if( IC3CONbits.ICBNE ) { sound_capture = true; }
		if( IC4CONbits.ICBNE ) { sound_capture = true; }
		
		if( sound_capture ) {
			__delay_ms(3);  // approx 1000mm
			led_off();
			
			for( i = 0 ; i < NUMBER_OF_MIC ; i++ ) {
   		  sound_capture_mic[i] = true;
			}
			if( IC1CONbits.ICBNE ) { ReadCapture1( &timer_value_mic[0] ); } else { sound_capture_mic[0] = false; error_microphone = true; }
			if( IC2CONbits.ICBNE ) { ReadCapture2( &timer_value_mic[1] ); } else { sound_capture_mic[1] = false; error_microphone = true; }
			if( IC3CONbits.ICBNE ) { ReadCapture3( &timer_value_mic[2] ); } else { sound_capture_mic[2] = false; error_microphone = true; }
			if( IC4CONbits.ICBNE ) { ReadCapture4( &timer_value_mic[3] ); } else { sound_capture_mic[3] = false; error_microphone = true; }
			for( i = 0 ; i < 4 ; i++ ) {
				ReadCapture1( &timer_value_mic_dummy );	
				ReadCapture2( &timer_value_mic_dummy );	
				ReadCapture3( &timer_value_mic_dummy );	
				ReadCapture4( &timer_value_mic_dummy );	
			}
			rs485_TX_enable();
			if( error_microphone ) {
				for( i = 0 ; i < NUMBER_OF_MIC ; i++ ) {
				  if( sound_capture_mic[i] == false ) { 
						sprintf(buf, "E\n" );
				  } else {
						sprintf(buf, "E %u\n", timer_value_mic[i] );
					}
			 		putsUART1( (unsigned int *)buf );					
				}
			} else {
				max_capture_value = 0;
				min_capture_value = MAX_TIMER_VALUE;
        for( i = 0 ; i < NUMBER_OF_MIC ; i++ ) {
				  if( max_capture_value < timer_value_mic[i] ) {
						max_capture_value = timer_value_mic[i];
					}
				  if( min_capture_value > timer_value_mic[i] ) {
						min_capture_value = timer_value_mic[i];
					}
				}
				if( (max_capture_value - min_capture_value) > (MAX_TIMER_VALUE/2) ){
					for( i = 0 ; i < NUMBER_OF_MIC ; i++ ) {
						if( timer_value_mic[i] < (MAX_TIMER_VALUE/2) ) {
							 timer_value_mic[i] += (MAX_TIMER_VALUE/2) + 1;
						} else {
							 timer_value_mic[i] -= (MAX_TIMER_VALUE/2);
						}
					}
				}
				min_capture_value = MAX_TIMER_VALUE;
	       for( i = 0 ; i < NUMBER_OF_MIC ; i++ ) {
					if( min_capture_value > timer_value_mic[i] ) {
						min_capture_value = timer_value_mic[i];
					}
  			}
				for( i = 0 ; i < NUMBER_OF_MIC ; i++ ) {
					sprintf(buf, "%u\n", timer_value_mic[i] - min_capture_value);
			  	putsUART1( (unsigned int *)buf );
				}
			}
			
			sprintf(buf, "EOR\n" );
			putsUART1( (unsigned int *)buf );
			rs485_RX_enable();			
  		__delay_ms(100);

			sound_capture = false;
	    error_microphone = false;
			CloseCapture1();  // clear capture buffer 
			CloseCapture2();
			CloseCapture3();
			CloseCapture4();
			OpenCapture1(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
			OpenCapture2(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
			OpenCapture3(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
			OpenCapture4(IC_TIMER2_SRC | IC_EVERY_RISE_EDGE);
		}
 	}    

  return 0;
}

//----------- E N D ------------------------------------------------------------